#!/usr/bin/python
#
# CDA tool: reads multiple bulk_extractor histogram files and outputs:
# stoplist.txt - list of email addresses on more than 1/3 of the disks
# targets.txt  - list of email addresses not on stoplist and the # of drives on which they appear.

import os,sys,re
if os.getenv("DOMEX_HOME"): sys.path.append(os.getenv("DOMEX_HOME") + "/src/lib/") # add the library
sys.path.append("../lib/")      # add the library

def mycmp2(a,b):
    if a[1]>b[1]: return -1
    if a[1]<b[1]: return 1
    return 0


class mdc:
    def __init__(self):
        self.total_drives = 0   # total drives processed
        self.email_drives = {}  # key=email / value=Set of drives on which it is present
        self.email_re = re.compile("(.*): (\d)")

    def add_file(self,fn):
        self.total_drives += 1
        for line in open(fn,"r"):
            m = self.email_re.search(line)
            if m:
                email = m.group(1).lower() # just work with lowercase
                email = email.replace("%20","")
                count = m.group(2)
                if email not in self.email_drives: self.email_drives[email] = set()
                self.email_drives[email].add(fn)

    def email_max(self):
        ret = 0
        for (e,dlist) in self.email_drives.iteritems():
            ret = max(ret,len(dlist))
        return ret
        

    def generate_stop_list(self,output_fn):
        emax = self.email_max()
        print "total drives:",self.total_drives
        print "max drives per email:",emax
        top_third = emax * 1/3

        stoplist = set()
        out = []
        for (e,dlist) in self.email_drives.iteritems():
            if len(dlist) > top_third:
                out.append((e,len(dlist)))
                stoplist.add(e)
        self.stoplist = stoplist

        out.sort(mycmp2)
        f = open(output_fn,"w")
        for (email,count) in out:
            f.write("%s\t%d\n" % (email,count))
        f.close()

    def generate_datasphere(self,fn):
        dsph = []
        for (e,dlist) in self.email_drives.iteritems():
            if e not in self.stoplist:
                count = len(dlist)
                dsph.append((e,count))
        dsph.sort()
        f = open(fn,"w")
        for (e,count) in dsph:
            f.write("%s\t%d\n" % (e,count))
        f.close()
        self.dsph = dsph

    def paircount(self,fn):
        paircount = {}
        for (e,dlist) in self.email_drives.iteritems():
            if e in self.stoplist: continue
            if len(dlist)==2:
                for drive in dlist:
                    paircount[drive] = paircount.get(drive,0)+1
        for (drive,count) in paircount.iteritems():
            print drive,count
        

    def correlate(self,fn):
        cda_tally = {}          # the score for each drive
        drive_emails = {}       # for each drive, the correlated emails
        for line in open(fn,"r"):
            m = self.email_re.search(line)
            if m:
                email = m.group(1).lower() # just work with lowercase
                email = email.replace("%20","")
                count = m.group(2)
                if email in self.stoplist: continue
                if email not in self.email_drives: continue
                drives = self.email_drives[email]
                if len(drives)==1: continue
                for drive in drives:
                    cda_tally[drive] = cda_tally.get(drive,0) + 1.0 / len(drives)
                    if drive not in drive_emails: drive_emails[drive] = set()
                    drive_emails[drive].add(email)
                    
        drives = cda_tally.keys()
        drive_scores = []
        for drive in drives:
            drive_scores.append((drive,cda_tally[drive]))
        drive_scores.sort(mycmp2)
        for (drive,drive_score) in drive_scores:
            print drive,drive_score
            correlated_scores = []
            for correlating_email in drive_emails[drive]:
                correlated_scores.append((correlating_email,1.0 / len(self.email_drives[correlating_email])))
            correlated_scores.sort(mycmp2)
            for (email,score) in correlated_scores:
                print "    ",email,score
            print ""
                

if(__name__=="__main__"):
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option("-c","--corr",dest="target", help="Perform correlation")
    parser.add_option("-d","--debug",action="store_true")
    (options,args) = parser.parse_args()

    mymdc = mdc()

    for fn in args:
        mymdc.add_file(fn)

    mymdc.generate_stop_list("stoplist.txt")
    mymdc.generate_datasphere("datasphere.txt")
    #mymdc.paircount("paircount.txt")

    if options.target:
        mymdc.correlate(options.target)

