import java.io.*;
import java.util.*;	// map, set

%%

%class BinaryLexer
%8bit
%caseless
%int

%{

  public MapCollection mc;	

  @SuppressWarnings("fallthrough")

  private String unicode16_to_string(String buf) {
    int new_length = buf.length()/2;
    StringBuilder new_string = new StringBuilder(new_length);
    for (int i = 0; i<new_length; i++) {
      new_string.append(buf.charAt(i*2));
    }
    return new_string.toString();
  }

  boolean at_eof(){
    return zzAtEOF;
  }

%}

printable_char = [\x20-\x7E]
u_printable_char = ([\x20-\x7E]\0)
st = [ \t]
u_st = ([ \t]\0)
stn = [ \t\n]
u_stn = ([ \t\n]\0)

year = ((19[6-9][0-9])|(200[0-9]))
u_year = ((1\09\0[6-9]\0[0-9]\0)|(2\00\00\0[0-9\0]))
day_of_week = (Mon|Tue|Wed|Thu|Fri|Sat|Sun)
u_day_of_week = (M\0o\0n\0|T\0u\0e\0|W\0e\0d\0|T\0h\0u\0|F\0r\0i\0|S\0a\0t\0|S\0u\0n\0)
month = (Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)
u_month = (J\0a\0n\0|F\0e\0b\0|M\0a\0r\0|A\0p\0r\0|M\0a\0y\0|J\0u\0n\0|J\0u\0l\0|A\0u\0g\0|S\0e\0p\0|O\0c\0t\0|N\0o\0v\0|D\0e\0c\0)
time_zone = (UT|GMT|EST|EDT|CST|CDT|MST|MDT|PST|PDT|Z|A|M|N|Y)
u_time_zone = (U\0T\0|G\0M\0T\0|E\0S\0T\0|E\0D\0T\0|C\0S\0T\0|C\0D\0T\0|M\0S\0T\0|M\0D\0T\0|P\0S\0T\0|P\0D\0T\0|Z\0|A\0|M\0|N\0|Y\0)
top_level_domain = (AC|AD|AE|AERO|AF|AG|AI|AL|AM|AN|AO|AQ|AR|ARPA|AS|ASIA|AT|AU|AW|AX|AZ|BA|BB|BD|BE|BF|BG|BH|BI|BIZ|BJ|BL|BM|BN|BO|BR|BS|BT|BV|BW|BY|BZ|CA|CAT|CC|CD|CF|CG|CH|CI|CK|CL|CM|CN|CO|COM|COOP|CR|CU|CV|CX|CY|CZ|DE|DJ|DK|DM|DO|DZ|EC|EDU|EE|EG|EH|ER|ES|ET|EU|FI|FJ|FK|FM|FO|FR|GA|GB|GD|GE|GF|GG|GH|GI|GL|GM|GN|GOV|GP|GQ|GR|GS|GT|GU|GW|GY|HK|HM|HN|HR|HT|HU|ID|IE|IL|IM|IN|INFO|INT|IO|IQ|IR|IS|IT|JE|JM|JO|JOBS|JP|KE|KG|KH|KI|KM|KN|KP|KR|KW|KY|KZ|LA|LB|LC|LI|LK|LR|LS|LT|LU|LV|LY|MA|MC|MD|ME|MF|MG|MH|MIL|MK|ML|MM|MN|MO|MOBI|MP|MQ|MR|MS|MT|MU|MUSEUM|MV|MW|MX|MY|MZ|NA|NAME|NC|NE|NET|NF|NG|NI|NL|NO|NP|NR|NU|NZ|OM|ORG|PA|PE|PF|PG|PH|PK|PL|PM|PN|PR|PRO|PS|PT|PW|PY|QA|RE|RO|RS|RU|RW|SA|SB|SC|SD|SE|SG|SH|SI|SJ|SK|SL|SM|SN|SO|SR|ST|SU|SV|SY|SZ|TC|TD|TEL|TF|TG|TH|TJ|TK|TL|TM|TN|TO|TP|TR|TRAVEL|TT|TV|TW|TZ|UA|UG|UK|UM|US|UY|UZ|VA|VC|VE|VG|VI|VN|VU|WF|WS|YE|YT|YU|ZA|ZM|ZW)

u_tld1 = (A\0C\0|A\0D\0|A\0E\0|A\0E\0R\0O\0|A\0F\0|A\0G\0|A\0I\0|A\0L\0|A\0M\0|A\0N\0|A\0O\0|A\0Q\0|A\0R\0|A\0R\0P\0A\0|A\0S\0|A\0S\0I\0A\0|A\0T\0|A\0U\0|A\0W\0|A\0X\0|A\0Z\0|B\0A\0|B\0B\0|B\0D\0|B\0E\0|B\0F\0|B\0G\0|B\0H\0|B\0I\0|B\0I\0Z\0|B\0J\0|B\0L\0|B\0M\0|B\0N\0|B\0O\0|B\0R\0|B\0S\0|B\0T\0|B\0V\0|B\0W\0|B\0Y\0|B\0Z\0|C\0A\0|C\0A\0T\0|C\0C\0|C\0D\0|C\0F\0|C\0G\0|C\0H\0|C\0I\0|C\0K\0|C\0L\0|C\0M\0|C\0N\0|C\0O\0|C\0O\0M\0|C\0O\0O\0P\0|C\0R\0|C\0U\0|C\0V\0|C\0X\0|C\0Y\0|C\0Z\0)
u_tld2 = (D\0E\0|D\0J\0|D\0K\0|D\0M\0|D\0O\0|D\0Z\0|E\0C\0|E\0D\0U\0|E\0E\0|E\0G\0|E\0H\0|E\0R\0|E\0S\0|E\0T\0|E\0U\0|F\0I\0|F\0J\0|F\0K\0|F\0M\0|F\0O\0|F\0R\0|G\0A\0|G\0B\0|G\0D\0|G\0E\0|G\0F\0|G\0G\0|G\0H\0|G\0I\0|G\0L\0|G\0M\0|G\0N\0|G\0O\0V\0|G\0P\0|G\0Q\0|G\0R\0|G\0S\0|G\0T\0|G\0U\0|G\0W\0|G\0Y\0|H\0K\0|H\0M\0|H\0N\0|H\0R\0|H\0T\0|H\0U\0|I\0D\0|I\0E\0|I\0L\0|I\0M\0|I\0N\0|I\0N\0F\0O\0|I\0N\0T\0|I\0O\0|I\0Q\0|I\0R\0|I\0S\0|I\0T\0|J\0E\0|J\0M\0|J\0O\0|J\0O\0B\0S\0|J\0P\0|K\0E\0|K\0G\0|K\0H\0|K\0I\0|K\0M\0|K\0N\0|K\0P\0|K\0R\0|K\0W\0|K\0Y\0|K\0Z\0)
u_tld3 = (L\0A\0|L\0B\0|L\0C\0|L\0I\0|L\0K\0|L\0R\0|L\0S\0|L\0T\0|L\0U\0|L\0V\0|L\0Y\0|M\0A\0|M\0C\0|M\0D\0|M\0E\0|M\0F\0|M\0G\0|M\0H\0|M\0I\0L\0|M\0K\0|M\0L\0|M\0M\0|M\0N\0|M\0O\0|M\0O\0B\0I\0|M\0P\0|M\0Q\0|M\0R\0|M\0S\0|M\0T\0|M\0U\0|M\0U\0S\0E\0U\0M\0|M\0V\0|M\0W\0|M\0X\0|M\0Y\0|M\0Z\0|N\0A\0|N\0A\0M\0E\0|N\0C\0|N\0E\0|N\0E\0T\0|N\0F\0|N\0G\0|N\0I\0|N\0L\0|N\0O\0|N\0P\0|N\0R\0|N\0U\0|N\0Z\0|O\0M\0|O\0R\0G\0|P\0A\0|P\0E\0|P\0F\0|P\0G\0|P\0H\0|P\0K\0|P\0L\0|P\0M\0|P\0N\0|P\0R\0|P\0R\0O\0|P\0S\0|P\0T\0|P\0W\0|P\0Y\0)
u_tld4 = (Q\0A\0|R\0E\0|R\0O\0|R\0S\0|R\0U\0|R\0W\0|S\0A\0|S\0B\0|S\0C\0|S\0D\0|S\0E\0|S\0G\0|S\0H\0|S\0I\0|S\0J\0|S\0K\0|S\0L\0|S\0M\0|S\0N\0|S\0O\0|S\0R\0|S\0T\0|S\0U\0|S\0V\0|S\0Y\0|S\0Z\0|T\0C\0|T\0D\0|T\0E\0L\0|T\0F\0|T\0G\0|T\0H\0|T\0J\0|T\0K\0|T\0L\0|T\0M\0|T\0N\0|T\0O\0|T\0P\0|T\0R\0|T\0R\0A\0V\0E\0L\0|T\0T\0|T\0V\0|T\0W\0|T\0Z\0|U\0A\0|U\0G\0|U\0K\0|U\0M\0|U\0S\0|U\0Y\0|U\0Z\0|V\0A\0|V\0C\0|V\0E\0|V\0G\0|V\0I\0|V\0N\0|V\0U\0|W\0F\0|W\0S\0|Y\0E\0|Y\0T\0|Y\0U\0|Z\0A\0|Z\0M\0|Z\0W\0)

u_top_level_domain = ({u_tld1}|{u_tld2}|{u_tld3}|{u_tld4})

message_id = (Message-ID:)
u_message_id = (M\0e\0s\0s\0a\0g\0e\0-\0I\0D\0:\0)
subject = (Subject:)
u_subject = (S\0u\0b\0j\0e\0c\0t\0:\0)
cookie = (Cookie:)
u_cookie = (C\0o\0o\0k\0i\0e\0:\0)
http = (https?:)
u_http = (h\0t\0t\0p\0(s\0)?:\0)


%%


/* RFC date */
{day_of_week},{stn}+[0-9]{1,2}{stn}+{month}{stn}+{year}{stn}+[0-2][0-9]:[0-5][0-9]:[0-5][0-9]{stn}+([+-][0-2][0-9][0314][05]|{time_zone})
            { mc.add_token(yytext(), mc.date_map); }

/* unicode_16 RFC date */
{u_day_of_week},{u_stn}+([0-9]\0){1,2}{u_stn}+{u_month}{u_stn}+{u_year}{u_stn}+[0-2]\0[0-9]\0:\0[0-5]\0[0-9]\0:\0[0-5]\0[0-9]\0{u_stn}+([+-]\0[0-2]\0[0-9]\0[0314]\0[05]\0|{u_time_zone})
            { mc.add_token(unicode16_to_string(yytext()), mc.date_map); }

/* message ID: [ \t\n]?<.*> */
{message_id}{stn}?<.*>
            { mc.add_token(yytext(), mc.message_id_map); }

/* unicode_16 message ID: [ \t\n]?<.*> */
{u_message_id}{u_stn}?\0<\0.\0*\0>\0
            { mc.add_token(unicode16_to_string(yytext()), mc.message_id_map); }

/* subject: [ \t]?({PC}{1,80}) */
{subject}{st}?{printable_char}{1,80}
            { mc.add_token(yytext(), mc.subject_map); }

/* unicode_16 subject: [ \t]?({PC}{1,80}) */
{u_subject}{u_st}?\0{u_printable_char}{1,80}
            { mc.add_token(unicode16_to_string(yytext()), mc.subject_map); }

/* cookie: [ \t]?({PC}{1,80}) */
{cookie}{st}?{printable_char}{1,80}
            { mc.add_token(yytext(), mc.cookie_map); }

/* unicode_16 cookie: [ \t]?({PC}{1,80}) */
{u_cookie}{u_st}?\0{u_printable_char}{1,80}
            { mc.add_token(unicode16_to_string(yytext()), mc.cookie_map); }

/* email */
[a-zA-Z0-9][a-zA-Z0-9._%\-+]{1,128}@[a-zA-Z0-9._%\-]{1,128}\.{top_level_domain}/[^a-zA-Z]
            { String email = yytext();
              mc.add_token(email, mc.email_map);
              String domain_name = email.substring(email.indexOf("@")+1, email.length());
              mc.add_token(domain_name, mc.domain_name_map);
            }

/* unicode_16 email */
[a-zA-Z0-9]\0([a-zA-Z0-9._%\-+]\0){1,128}@\0([a-zA-Z0-9._%\-]\0){1,128}\.\0{u_top_level_domain}/[^a-zA-Z]|([^][^\0])
            { String email = unicode16_to_string(yytext());
/* System.out.println("email.a: "+yytext()); */
/* System.out.println("email.b: "+unicode16_to_string(yytext())); */
              mc.add_token(email, mc.email_map);
              String domain_name = email.substring(email.indexOf("@")+1, email.length());
              mc.add_token(domain_name, mc.domain_name_map);
            }

/* URL: https?:[a-zA-Z0-9_%/\-+@:=&\?#~.]{1,128}/[^a-zA-Z0-9_%\/\-+@:=&\?#~.] */
{http}[a-zA-Z0-9_%/\-+@:=&\?#~.]{1,128}/[^a-zA-Z0-9_%\/\-+@:=&\?#~.]
            { mc.add_token(yytext(), mc.url_map); }

/* unicode_16 URL: */
{u_http}([a-zA-Z0-9_%/\-+@:=&\?#~.]\0){1,128}/[^a-zA-Z0-9_%\/\-+@:=&\?#~.]|([^][^\0])
{
/* System.out.println("url.a: "+yytext()); */
/* System.out.println("url.b: "+unicode16_to_string(yytext())); */
              mc.add_token(unicode16_to_string(yytext()), mc.url_map); }

[^]     { }
