/**
 *
 * @author Bruce Allen, original author
 * @author Simson Garfinkel, additional work
 */


import java.io.*;
import java.util.*;	// map, set

public class MapCollection {
    Map<String, Integer> date_map = new TreeMap<String, Integer>();
    Map<String, Integer> message_id_map = new TreeMap<String, Integer>();
    Map<String, Integer> subject_map = new TreeMap<String, Integer>();
    Map<String, Integer> cookie_map = new TreeMap<String, Integer>();
    Map<String, Integer> email_map = new TreeMap<String, Integer>();
    Map<String, Integer> url_map = new TreeMap<String, Integer>();
    Map<String, Integer> domain_name_map = new TreeMap<String, Integer>();

    Set<String> suppress_set = new TreeSet<String>();

    void read_suppress_set_from_file(String fname){
	try {
	    BufferedReader br = new BufferedReader(new FileReader(fname));
	    String line;
	    while ((line = br.readLine()) != null) { // while loop begins here
		suppress_set.add(line);
		System.out.println(line);
	    }
	    System.out.println("Lines in suppress file: "+suppress_set.size());
	} 
	catch (IOException e) {
	    System.err.println("Error: " + e);
	}
    }

    void add_token(String key, Map<String, Integer> map) {
	Integer value = map.get(key);
	if (value == null) {
	    // the key is new
	    map.put(key, new Integer(1));
	} else {
	    // increment existing key's value
	    map.put(key, new Integer(value.intValue()+1));
	}
    }

    void print_report() {
	print_histogram("Top 10 email addresses:", email_map, 10);
	print_histogram("Top 10 email domains:", domain_name_map, 10);
	print_histogram("Top 10 URLs:", url_map, 10);
	print_histogram("All email addresses:", email_map, 0);
	print_histogram("All email domains:", domain_name_map, 0);
	print_histogram("All URLs:", url_map, 0);

	print_histogram("Top 10 dates:", date_map, 10);
	print_histogram("Top 10 message IDs:", message_id_map, 10);
	print_histogram("Top 10 subject lines:", subject_map, 10);
	print_histogram("Top 10 cookies:", cookie_map, 10);
    }

    /** Print the histogram
     */

    void print_histogram(String title, Map<String, Integer> map, int max) {

	if (max == 0) max = Integer.MAX_VALUE;	// limit the number of keys identified

	// create properly sortable set of token objects for printing
	TreeSet<TokenCounter> printable_set = new TreeSet<TokenCounter>();

	// loop through map's keys to find keys that occur the most frequently
	Iterator<String> map_iterator = map.keySet().iterator();
	while (map_iterator.hasNext()) {
	    String key = map_iterator.next();
	    int value = map.get(key).intValue();

	    if (suppress_set.contains(key)){
		continue;
	    }

	    // maybe add the entry
	    if (printable_set.size() < max) {
		// just add the entry; we don't have max number of items yet.
		printable_set.add(new TokenCounter(key, value));
	    } else {
		// if the entry is larger than the lowest value: remove lowest and add this one
		int lowest_value = printable_set.last().count;
		if (value > lowest_value) {
		    printable_set.remove(printable_set.last());
		    printable_set.add(new TokenCounter(key, value));
		}
	    }
	}

	// print the set
	System.out.println(title+"\n====================");
	Iterator<TokenCounter> set_iterator = printable_set.iterator();
	while (set_iterator.hasNext()) {
	    System.out.println(set_iterator.next());
	}
	System.out.println();
    }
}
