/*
 *
 * Feature Extraction Framework...
 *
 */

/* TTD:  handle badfalg! */


#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif

#include <sys/time.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <openssl/md5.h>

#include <algorithm>
#include <cstdlib>
#include <vector>
#include <string>
#include <set>

#include <iostream>
#include <fstream>

#include <afflib/afflib.h>
#include <afflib/afflib_i.h>
#include <openssl/sha.h>

#include "bulk_extractor.h"
#include "histogram.h"
#include "bloom.h"

NSRLBloom *featureBloom=0;		// in-memory bloom
NSRLBloom *outputBloom=0;     // found features; BF must already exist

const char *outfile_fn = 0;
using namespace std;

histogram email_addresses;			// email addresses
histogram email_domains;			// 
histogram urls;


/* We keep a map of MD5s that have been seen for each page, so that a page is not processed twice */
struct md5 {
    char buf[16];
};

struct lmd5
{
    bool operator()(struct md5 m1, struct md5 m2) const
    {
	return memcmp(m1.buf,m2.buf, 16) < 0;
    }
};


const char *progname=0;
int   debug=0;
int   verbose=0;

/* The structure that is passed from the extractor back to this routine */
struct notify_params {
    unsigned char *base;
    long long pos0;			// offset
};

string unicode16_to_string(const char *buf, size_t len)
{

    // unicode, so strip out the null bytes
    char *buf2 = (char *)malloc(len/2);
    for(u_int i=0; i<len/2; i++) {
        buf2[i] = buf[i*2];
    }
    string new_string = string(buf2, len/2);

//cout << "bulk_extractor buf: " << len << ": ";
//for (u_int j=0; j<len; j++)
//    cout << buf[j];
//cout << "\n";
//cout << "bulk_extractor.string: " << new_string << "\n";

    free(buf2);
    return new_string;
}


/** turn an email address into a domain */
string just_domain(string &email)
{
    size_t pos = email.find('@');
    if(pos<0) return "";
    return email.substr(pos+1);
}




/*
 * niceprint(char *buf,int len):
 * Prints some of a buffer, but only printable characters...
 */
void niceprint(unsigned char *buf,int len)
{
    while(len-->0){
	if(*buf == '|' || !isprint(*buf)){
	    putchar('.');
	}
	else {
	    putchar(*buf);
	}
	buf++;
    }
}


/*
 * fef notify function.
 * Prints what was found with printable context before and after.
 */

void feature_found(const char *code,const  char *buf,int len,int pos,void *tag)
{
    if(debug){
	printf("feature_found(%s,%p,%d,%d,%p)\n",code,buf,len,pos,tag);
	fflush(stdout);
    }

    struct notify_params *np = (struct notify_params *)tag;
    if(len>1024) return;		// don't use the big features

    if(verbose){
	fputs(code,stdout);			// output the code
	putchar('|');			// delim
	niceprint(np->base+pos-20,20);	// 20 characters of context before
	putchar('|');
	niceprint((unsigned char*)buf,len);	// print what matched
	putchar('|');
	niceprint(np->base+pos+len,20);	// 20 characters of context afterwards
	printf("  (pos=%qd)\n",np->pos0+pos); // where it was in the file
    }

    if(code[0]=='E' || code[0]=='e') {
	string email;
	string domain;

        if (code[0]=='e') {
            // unicode, so strip out the null bytes
            email = unicode16_to_string(buf, len);
        } else {
            // not unicode, so take email directly
	    email = string(buf,len);
        }

	domain = just_domain( email );

	/**
	 * If we have a featureBloom, only process the features that are already in the bloom
	 * In this way, we only add names that are in twice
	 */
	bool emailPresent = featureBloom==0 ? true : featureBloom->queryString(email);
	bool domainPresent = featureBloom==0 ? true : featureBloom->queryString(domain);

	if(emailPresent==false) featureBloom->addString(email);
	if(domainPresent==false) featureBloom->addString(domain);

	if(featureBloom){
	    if(emailPresent){
		email_addresses.addIfNotPresent(email);
		email_addresses.add(email);
	    }
	    if(domainPresent){
		email_domains.addIfNotPresent(domain);
		email_domains.add(domain);
	    }
	}
	else {
	    email_addresses.add(email);
	    email_domains.add(domain);
	}
    }

    if(code[0]=='U' || code[0]=='u') {
	string url;

        if (code[0]=='u') {
            // unicode, so strip out the null bytes
            url = unicode16_to_string(buf, len);
        } else {
            // not unicode, so take email directly
	    url = string(buf,len);
        }

	urls.add(url);
    }
}


void usage()
{
    fprintf(stderr,"Usage: %s [options] <file>\n", progname);
    fprintf(stderr,"  runs bulk extractor and outputs to stdout a summary of what was found where\n");
    fprintf(stderr,"             -d   debug\n");
    fprintf(stderr,"             -v   verbose - print each feature as it is found\n");
    fprintf(stderr,"             -b bloom.bf  - Also stores SHA1 of features in bloom.bf\n");
    fprintf(stderr,"             -o outfile   - specifies output file\n");
    fprintf(stderr,"             -s page#     - start on page number ##\n");
    fprintf(stderr,"             -m           - Low memory: don't print extracts with n==1\n");
    fprintf(stderr,"\n");
    //fprintf(stderr,"Installed plug-ins:\n");
    exit(1);
}

/* Return true if this is all blank */
bool all_blank(const unsigned char *buf,int len)
{
    while(len>0){
	if(*buf!=0) return false;
	buf++;
	len--;
    }
    return true;
}

void output_results(string input_fn,int start_page,int page)
{
    string bak = string(outfile_fn) + ".bak";
    ofstream o;
    FrequencyReport *fr;
    time_t t = time(0);

    rename(outfile_fn,bak.c_str());	// create the bakup file

    o.open(outfile_fn);
    o << "Input file: " << input_fn << "\n";
    o << "Starting page number: " << start_page << "\n";
    o << "Last processed page number: " << page << "\n";
    o << "Time: " << ctime(&t);

    o << "\n";
    o << "Top 10 email addresses:\n" ;
    o << "=======================\n" ; 

    fr  = email_addresses.makeReport(10);
    o << *fr;
    delete fr;

    o << "\n";
    o << "Top 10 email domains:\n";
    o << "=====================\n" ; 
    fr = email_domains.makeReport(10);
    o << *fr;
    delete fr;

    o << "\n";
    o << "Top 10 URLs:\n";
    o << "=====================\n" ; 
    fr = urls.makeReport(10);
    o << *fr;
    delete fr;
    
	

    o << "\n";
    o << "All email addresses:\n";
    o << "====================\n" ; 
    fr = email_addresses.makeReport();
    o << *fr;
    delete fr;

    o << "\n";
    o << "All email domains:\n";
    o << "==================\n" ; 
    fr = email_domains.makeReport();
    o << *fr;
    delete fr;

    o << "\n";
    o << "All urls:\n";
    o << "==================\n" ; 
    fr = urls.makeReport();
    o << *fr;
    delete fr;

    o.close();
}

AFFILE *af=0;
void status(int n)
{
    printf("af->pos=%"PRId64"\n",af->pos);
}

/* This code is similar to afcat. It makes a list of all the
 * data pages and then processes each one.
 */

int main(int argc,char **argv)
{
    int start_page = 0;
    struct notify_params np;
    memset(&np,0,sizeof(np));
    
    progname = argv[0];
    int ch;
    int opt_all = 1;
    while ((ch = getopt(argc, argv, "mdh?vbo:s:")) != -1) {
	switch (ch) {
	default:
	case 'm':
	    opt_all = 0;
	    break;
	case 'b':
	    outputBloom = new NSRLBloom();
	    if(outputBloom->open(optarg,O_RDWR)){
		fprintf(stderr,"Cannot open output bloomfile:");
		exit(1);
	    }
	    break;
	case 'd': debug++; break;
	case 'v': verbose++; break;
	case 'o': outfile_fn = optarg;break;
	case 's':
	    start_page = atoi(optarg);
	    fprintf(stderr,"Will start on page %d\n",start_page);
	    break;
	case 'h':
	case '?':
	    usage();
	    break;
	}
    }

    if(outfile_fn==0){
	fprintf(stderr,"error: -o outfile must be specified\n");
	usage();
    }
    if(access(outfile_fn,F_OK)==0) errx(1,"%s: file exists. Will not overwrite existing files\n",outfile_fn);


    if(!opt_all){
	featureBloom = new NSRLBloom();
	featureBloom->create(0,128,30,4,"in-memory bloom filter for feature suppression");
    }

    argc -= optind;
    argv += optind;

    if ( argc !=1 ){
	usage();
    }

    af = af_open(*argv,O_RDONLY,0666);
    if(!af){
	perror(*argv);
	exit(1);
    }
    if(af_cannot_decrypt(af)) errx(1,"Cannot decrypt %s: encryption key not set?",*argv);


    scanner_t (*test_plugin) = scan_email;

    /* Get a list of all the pages */
    vector<int> pagelist;

    af_rewind_seg(af);			// start at the beginning
    char segname[AF_MAX_NAME_LEN];
    memset(segname,0,sizeof(segname));
    while(af_get_next_seg(af,segname,sizeof(segname),0,0,0)==0){
	int page_num = af_segname_page_number(segname);
	if(page_num>=0) pagelist.push_back(page_num);
    }
    sort(pagelist.begin(),pagelist.end());

    
    /* Now process each segment */
    const int window_margin = 16;	// extra size on both sides of segment
    const int margin_2x = window_margin * 2;
    unsigned char *buf = (unsigned char *)calloc(af->image_pagesize + margin_2x,1); 
    unsigned char *md5buf = (unsigned char *)malloc(af->image_pagesize + margin_2x);
    
#ifdef SIGINFO
    signal(SIGINFO,status);
#endif
    
    set <struct md5,lmd5> seen_md5s;

    for(vector<int>::const_iterator i = pagelist.begin(); i!= pagelist.end(); i++){
	int pagenum = *i;

	if(pagenum < start_page) continue;

	printf("Processing page %d out of %d...\n",pagenum,pagelist.back());
	fflush(stdout);

	size_t bytes    = af->image_pagesize;
	size_t rawbytes = af->image_pagesize;

	/* Calculate the MD5 of the page to see if we have read it before */
	if(af_get_page_raw(af,pagenum,0,md5buf,&rawbytes)){
	    warn("Could not read raw page %d",pagenum);
	    goto clean_up;		// failed to get raw page?
	}

	if(debug) fprintf(stderr,"Page %d  len=%zu\n",pagenum,rawbytes);

	/* Remember that we have seen this md5 */
	struct md5 this_md5;
	MD5(md5buf,rawbytes,(unsigned char *)this_md5.buf);

	if(seen_md5s.find(this_md5)!=seen_md5s.end()){
	    if(debug) printf("Seen page %d before.\n",pagenum);
	    goto clean_up;
	}
	seen_md5s.insert(this_md5);	// add the MD5 for this sector
 
	if(af_get_page(af,pagenum,buf+window_margin,&bytes)==0){
	    if(debug) printf("   bytes=%zu\n",bytes);
	    if(all_blank(buf+window_margin,bytes)){ // don't extract segments that are all blank
		if(debug) printf("  all blank!\n");
		goto clean_up;
	    }
	    
	    np.base = buf;
	    np.pos0 = pagenum * af->image_pagesize - window_margin;;

	    /* TK: Check for bad blocks */
	    try {
		(*test_plugin)(buf,bytes+window_margin,feature_found,(void *)&np);
	    }
	    catch ( exception &e ) {
		cout << "Exception: ";
		cout << e.what() << "\n";
		printf("Rerun starting at page %d\n",pagenum);
		break;
	    }
	    output_results(af_filename(af),start_page,pagenum);

	clean_up:
	    /* Now copy out the last window_margin bytes of context... */
	    memmove(buf,buf+window_margin+bytes-window_margin,window_margin);
	}
	if(debug) printf("   ...\n");
    }
    printf("                                           \n\n\n");
    (*test_plugin)(0,0,feature_found,(void *)&np); // clear it out
    af_close(af);

    if(outputBloom) delete outputBloom;
    if(featureBloom) delete featureBloom;


    /* Print usage information */
#ifdef HAVE_GETRUSAGE
    struct rusage ru;
    memset(&ru,0,sizeof(ru));
    if(getrusage(RUSAGE_SELF,&ru)==0){
	printf("# u: %d  s: %d\n",(int)ru.ru_utime.tv_sec,(int)ru.ru_stime.tv_sec);
	printf("# maxrss: %ld reclaims: %ld faults: %ld   swaps: %ld\n",
	       ru.ru_maxrss,ru.ru_minflt,ru.ru_majflt,ru.ru_nswap);
	printf("# inputs: %ld  outputs: %ld\n",ru.ru_inblock,ru.ru_oublock);
    }
#endif
}

