#include <iostream>
#include <map>
#include <vector>
#include <algorithm>

#include "histogram.h"

using namespace std;

/* Histogram class. The basis of a string-based correlator. Uses C++ STL for sorting
 * and string handling.
 */
		
ostream & operator << (ostream &os, const FrequencyReport &rep){
    for(FrequencyReport::const_iterator i = rep.begin(); i!=rep.end();i++){
	os << i->value << ": " << i->count << "\n";
    }
    return os;
}

FrequencyReport *histogram::makeReport() 
{
    FrequencyReport *rep = new FrequencyReport();
    for(map<string,int>::const_iterator i = db.begin();
	i != db.end();
	i++){
	ReportElement re(i->first,i->second);
	rep->push_back(re);
    }
    sort(rep->begin(),rep->end(),ReportElement::compare);
    return rep;
}

FrequencyReport *histogram::makeReport(int topN) 
{
    FrequencyReport *r2 = makeReport();	// gets a new report
    FrequencyReport::iterator i = r2->begin();
    while(topN>0 && i!=r2->end()){
	i++;
	topN--;
    }
    r2->erase(i,r2->end());
    return r2;
}


#ifdef TEST
main(int argc,char **argv)
{
    histogram h;

    printf("adding...\n");
    for(int j=1;j<10;j++){
	printf("j=%d\n",j);
	for(int i=0;i<10000;i+=j){
	    char buf[1024];
	    sprintf(buf,"number%d",i);
	    h.add(buf);
	}
    }
    printf("Generating report...\n");

    h.add(string("abdefghijklmnoop",3));

    h.add("this");
    h.add("is");
    h.add("a");
    h.add("test");
    h.add("this");
    h.add("yes sir");
    FrequencyReport *rep = h.makeReport();
    cout << *rep;
    delete rep;
}

#endif
